<?php
// $Horde: gollem/lib/Driver.php 06/18/2002 11:30:01 mike $

/*
 * The Gollem_Driver:: class provides a common abstracted interface to the
 * various filesystem connections.  It includes functions for adding, removing, and
 * renaming files & folders. 
 * 
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @author  Michael Varghese <mike.varghese@ascellatech.com>
 * @version <INSERT REVISION>
 * @since   Gollem
 * @package gollem
 */

class Gollem_Driver
{
    /** String holding the name of the current driver. */
    var $type = '';

    /** Hash holding the driver's additional parameters. */
    var $params = array();

    /**
     * Returns the name of the current driver type.
     *
     * @return          String containing the name of the current driver type.
     */
    function getType()
    {
        return $this->type;
    }

    /**
     * Returns the current driver's additional parameters.
     *
     * @return          Hash containing the driver's additional parameters.
     */
    function getParams()
    {
        return $this->params;
    }
	
    /**
     * Attempts to return a reference to a concrete Gollem_Driver instance
     * based on $driver. It will only create a new instance if no
     * Gollem_Driver instance with the same parameters currently exists.
     *
     * This method must be invoked as: $var = &Gollem_Driver::singleton()
     *
     * @param $driver   The type of concrete Gollem_Driver subclass to return.
     *                  This is based on the storage driver ($driver). The
     *                  code is dynamically included.
     * @param $params   A hash containing additional parameters for the
     *                  subclass.
     *
     * @return          The concrete Gollem_Driver reference, or false on an
     *                  error.
     */
	
	function &singleton($driver, $params)
	{
		static $instances;

        if (!isset($instances)) $instances = array();

        $signature = md5(strtolower($driver) . '][' . @implode('][', $params));
        if (!isset($instances[$signature])) {
            $instances[$signature] = &Gollem_Driver::factory($driver, $params);
        }
        return $instances[$signature];
	}
	/**
     * Attempts to return a concrete Gollem_Driver instance based on $driver.
     *
     * @param $driver   The type of Golem_Driver subclass to return.  The
     *                  code is dynamically included.
     * @param $params   Hash containing additional paramters to be passed to
     *                  the subclass's constructor.
     *
     * @return          The newly created concrete Gollem_Driver instance, or
     *                  false on an error.
     */
	 function &factory($driver, $params = array())
    {
        $driver = strtolower(basename($driver));
        @include_once dirname(__FILE__) . '/Drivers/' . $driver . '.php';
        $class = 'Gollem_Driver_' . $driver;
        if (class_exists($class)) {
            return new $class($params);
        } else {
            Horde::fatal(new PEAR_Error(sprintf(_("Unable to load the definition of %s."), $class)), __FILE__, __LINE__);
        }
    }
	
	function subdirectory($base, $dir)
    {
        if (empty($base)) {
            return $dir;
        } else {
            return $base . '/' . $dir;
        }
    }
	
	/**
     * Generate an URL to the logout screen that includes any known
     * information, such as username, server, etc., that can be filled
     * in on the login form.
     *
     * @param string $uri The page that will process the logout.
     *
     * @return string     The $uri with parameters added.
     */	
	function logoutURL ($uri) {
        global $conf, $gollem;

        if (!empty($gollem['fileuser']))
            $uri .= '&fileuser=' . urlencode($gollem['fileuser']);
        elseif (!empty($GLOBALS['imapuser']))
            $uri .= '&user=' . urlencode($GLOBALS['imapuser']);

        if ($conf['server']['server_list'] && !empty($GLOBALS['server']))
            $uri .= '&server=' . urlencode($GLOBALS['server']);
        elseif (!empty($gollem['server']))
            $uri .= '&server=' . urlencode($gollem['server']);

        if (!empty($gollem['port']))
            $uri .= '&port=' . urlencode($gollem['port']);
        if (!empty($gollem['language']))
            $uri .= '&language=' . urlencode($gollem['language']);

        return $uri;
    }
	
	/**
	 * Create a session through the specified backend.
	 */
	function createSession()
	{
		//abstract
	}
	
	/**
	 * Checks to see if a gollem session is setup or not.
	 */
	function setupSession()
	{
		//abstract
	}
	
	/**
	 * Authenticates the user for the current session. 
	 */
	function authenticate()
	{
		//abstract
	}
	
	/**
	 * Stores the file list within an array. 
	 */
	function getFileList($dir='')
	{
		//abstract
	}
	
	/**
	 * Sorts file list by date in order given by global prefs variable 
	 */
	function sortDate($a, $b)
	{
		//abstract
	}

	/**
	 * Sorts file list by size in order given by global prefs variable 
	 */
	function sortSize($a, $b)
	{
		//abstract
	}
	
	/**
	 * Sorts file list by type in order given by global prefs variable 
	 */
	function sortType($a, $b)
	{
		//abstract
	}
	
	/**
	 * Sorts file list by name in order given by global prefs variable 
	 */
	function sortName($a, $b)
	{
		//abstract
	}
	
	/**
	 * Sets the current directory as the home directory.
	 */
	function setLabel()
	{
		//abstract
	}
	
	/**
     * Create a folder through the specified backend.
     */
	function createFolder($stream, $name)
	{
		//abstract
	}
	
	/**
     * Rename a folder or file through the specified backend.
     */
	function renameItem($old, $new)
	{
		//abstract
	}
	
	/**
     * Rename a folder through the specified backend.
     */
	function renameFolder($old, $new)
	{
		//abstract
	}
	
	/**
     * Rename a file through the specified backend.
     */
	 function renameFile($old, $new)
	 {
	 	//abstract
	 }
	
	/**
     * Delete a folder or file through the specified backend.
     */
	function deleteItem($name)
	{
		//abstract
	}
	
	/**
     * Delete a folder through the specified backend.
     */
	function deleteFolder($name)
	{
		//abstract
	}
	/**
     * Delete a file through the specified backend.
     */
	function deleteFile($name)
	{
		//abstract
	}
	
	/**
     * Modify the CHMOD of the file through the specified backend.
     */
	function modifyCHMOD($chmod, $name)
	{
		//abstract
	}
	
	/**
     * Uploads a file to the specified backend.
     */
	function uploadFile($name, $safe_file, $uploadFormat)
	{
		//abstract
	}
}

?>